//
//  DynamicButton.m
//  DynamicUIButton
//
/*Copyright (c) 2012 Brandon McQuilkin
 
 Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files (the "Software"), to deal in the Software without restriction, including without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is furnished to do so, subject to the following conditions:
 
 The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.
 
 THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */

#import "DynamicButton.h"

#define kDBStandardHeight 26.0
#define kDBWidthPaddingRatio 0.45
#define kDBHeightPaddingRatio 0.3
#define kDBImageTextPaddingRatio 0.25
#define kDBAfterProcessTimer 1.5
#define kDBFontKey @"Font"
#define kDBSizedCharacters @"1234567890QWERTYUIOPASDFGHJKLZXCVBNM"

@interface DynamicButton ()

- (NSDictionary *)preformPrameterCheck:(NSDictionary *)parameters forState:(NSString *)state;
- (CGRect)frameForState:(NSString *)state;
- (UIImage *)imageWithImage:(UIImage *)image scaledToSize:(CGSize)newSize;
- (void)toggleState;
- (void)setupLayers;
- (void)cancelConfirm;
- (void)lighten;
- (void)darken;

@end

@implementation DynamicButton

- (NSDictionary *)preformPrameterCheck:(NSDictionary *)parameters forState:(NSString *)state
{
    //Preform check to make sure parameters are consistent, insert defaults for missing state parameters
    NSMutableDictionary *tempDict = [parameters mutableCopy];
    if ([state isEqualToString:kDBStateInitial]) {
        if ([tempDict objectForKey:kDBParameterTitleKey] == nil) {
            [tempDict setObject:@"Initial State" forKey:kDBParameterTitleKey];
        }
        if ([tempDict objectForKey:kDBParameterTintKey] == nil) {
            [tempDict setObject:[UIColor colorWithRed:0.220 green:0.357 blue:0.608 alpha:1] forKey:kDBParameterTintKey];
        }
    }
    if ([state isEqualToString:kDBStateConfirm]) {
        if ([tempDict objectForKey:kDBParameterTitleKey] == nil) {
            [tempDict setObject:@"Are you sure?" forKey:kDBParameterTitleKey];
        }
        if ([tempDict objectForKey:kDBParameterTintKey] == nil) {
            [tempDict setObject:[UIColor colorWithRed:0.439 green:0.741 blue:0.314 alpha:1] forKey:kDBParameterTintKey];
        }
    }
    if ([state isEqualToString:kDBStateProcess]) {
        if ([tempDict objectForKey:kDBParameterTitleKey] == nil) {
            [tempDict setObject:@"Processing..." forKey:kDBParameterTitleKey];
        }
        if ([tempDict objectForKey:kDBParameterTintKey] == nil) {
            [tempDict setObject:[UIColor colorWithRed:0.176 green:0.569 blue:0.820 alpha:1] forKey:kDBParameterTintKey];
        }
    }
    if ([state isEqualToString:kDBStateSuccess]) {
        if ([tempDict objectForKey:kDBParameterTitleKey] == nil) {
            [tempDict setObject:@"Success ✔" forKey:kDBParameterTitleKey];
        }
        if ([tempDict objectForKey:kDBParameterTintKey] == nil) {
            [tempDict setObject:[UIColor colorWithRed:0.000 green:0.502 blue:0.000 alpha:1] forKey:kDBParameterTintKey];
        }
    }
    if ([state isEqualToString:kDBStateFailure]) {
        if ([tempDict objectForKey:kDBParameterTitleKey] == nil) {
            [tempDict setObject:@"Failure ✗" forKey:kDBParameterTitleKey];
        }
        if ([tempDict objectForKey:kDBParameterTintKey] == nil) {
            [tempDict setObject:[UIColor colorWithRed:0.694 green:0.184 blue:0.196 alpha:1] forKey:kDBParameterTintKey];
        }
    }
    if ([state isEqualToString:kDBStateDisabled]) {
        if ([tempDict objectForKey:kDBParameterTitleKey] == nil) {
            [tempDict setObject:@"DISABLED" forKey:kDBParameterTitleKey];
        }
        if ([tempDict objectForKey:kDBParameterTintKey] == nil) {
            [tempDict setObject:[UIColor colorWithWhite:0.85 alpha:1] forKey:kDBParameterTintKey];
        }
    }
    return tempDict;
}

- (CGRect)frameForState:(NSString *)state
{
    CGSize size = [[[parametersDictionary objectForKey:state] objectForKey:kDBParameterTitleKey] sizeWithFont:[parametersDictionary objectForKey:kDBFontKey]];
    CGRect currentFrame = self.frame;
    UIImage *tempImage = [parametersDictionary objectForKey:kDBIconKey];
    currentFrame.size.width = size.width + (self.frame.size.height * kDBWidthPaddingRatio) + tempImage.size.width + (tempImage.size.width * kDBImageTextPaddingRatio);
    return currentFrame;
}

- (UIImage *)imageWithImage:(UIImage *)image scaledToSize:(CGSize)newSize
{
    UIGraphicsBeginImageContextWithOptions(newSize, NO, 0.0);
    [image drawInRect:CGRectMake(0, 0, newSize.width, newSize.height)];
    UIImage *newImage = UIGraphicsGetImageFromCurrentImageContext();
    UIGraphicsEndImageContext();
    return newImage;
}

- (id)initWithParameters:(NSDictionary *)parameters andFrame:(CGRect)frameTemplate
{
    //Initalize
    self = [super initWithFrame:frameTemplate];
    if (self != nil) {
        //Preform checks on each non-nil state to prevent crashes
        NSMutableDictionary *tempParameters = [[NSMutableDictionary alloc] initWithDictionary:parameters];
        [tempParameters setObject:[self preformPrameterCheck:[parameters objectForKey:kDBStateInitial] forState:kDBStateInitial] forKey:kDBStateInitial]; //Always Check, everything is based off of this parameter
        if ([parameters objectForKey:kDBStateConfirm] != nil) {
            [tempParameters setObject:[self preformPrameterCheck:[parameters objectForKey:kDBStateConfirm] forState:kDBStateConfirm] forKey:kDBStateConfirm];
        }
        if ([parameters objectForKey:kDBStateProcess] != nil) {
            [tempParameters setObject:[self preformPrameterCheck:[parameters objectForKey:kDBStateProcess] forState:kDBStateProcess] forKey:kDBStateProcess];
        }
        if ([parameters objectForKey:kDBStateSuccess] != nil) {
            [tempParameters setObject:[self preformPrameterCheck:[parameters objectForKey:kDBStateSuccess] forState:kDBStateSuccess] forKey:kDBStateSuccess];
        }
        if ([parameters objectForKey:kDBStateFailure] != nil) {
            [tempParameters setObject:[self preformPrameterCheck:[parameters objectForKey:kDBStateFailure] forState:kDBStateFailure] forKey:kDBStateFailure];
        }
        if ([parameters objectForKey:kDBStateDisabled] != nil) {
            [tempParameters setObject:[self preformPrameterCheck:[parameters objectForKey:kDBStateDisabled] forState:kDBStateDisabled] forKey:kDBStateDisabled];
        }
        if ([parameters objectForKey:kDBToggleStyleKey] == nil) {
            [tempParameters setObject:kDBToggleLeft forKey:kDBToggleStyleKey];
        }
        
        //Set parameters
        parametersDictionary = tempParameters;
        
        //Set this so button will show animations
        self.layer.needsDisplayOnBoundsChange = YES;
        
        //Set default height if zero
        if (frameTemplate.size.height == 0) {
            frameTemplate.size.height = kDBStandardHeight;
        }
        
        //Set default font if nil
        if ([parametersDictionary objectForKey:kDBFontNameKey] == nil) {
            [parametersDictionary setObject:[UIFont boldSystemFontOfSize:14].fontName forKey:kDBFontNameKey];
        }
        
        //Calculate UIEdgeInsets if there is an icon and set the icon
        if ([parametersDictionary objectForKey:kDBIconKey] != nil) {
            UIImage *tempImage = [parametersDictionary objectForKey:kDBIconKey];
            CGFloat scaleRatio = ((frameTemplate.size.height - (frameTemplate.size.height * kDBHeightPaddingRatio)) * 0.8) / tempImage.size.height;
            UIImage *icon = [self imageWithImage:tempImage scaledToSize:CGSizeMake(tempImage.size.width * scaleRatio, tempImage.size.height * scaleRatio)];
            [parametersDictionary setObject:icon forKey:kDBIconKey];
            [self setImage:icon forState:UIControlStateNormal];
            self.imageEdgeInsets = UIEdgeInsetsMake(0, 0, 0, icon.size.height * kDBImageTextPaddingRatio);
            self.titleEdgeInsets = UIEdgeInsetsMake(0, icon.size.height * kDBImageTextPaddingRatio, 0, 0);
            self.imageView.contentMode = UIViewContentModeScaleAspectFit;
            self.imageView.frame = CGRectMake((self.frame.size.height * kDBHeightPaddingRatio), (self.frame.size.height * kDBHeightPaddingRatio), self.imageView.frame.size.width, self.imageView.frame.size.height);
            self.imageView.layer.shadowOffset = CGSizeMake(0, -1);
            self.imageView.layer.shadowColor = [UIColor blackColor].CGColor;
            self.imageView.layer.shadowOpacity = 0.5;
            self.imageView.layer.shadowRadius = 0;
            self.imageView.layer.masksToBounds = NO;
            self.adjustsImageWhenHighlighted = NO;
        }
        
        //Calculate font height and set font
        CGFloat fontFrameHeight = 0.0;
        CGFloat maximumFontPointSize = 0.0;
        while (fontFrameHeight <= frameTemplate.size.height - (frameTemplate.size.height * kDBHeightPaddingRatio)) {
            fontFrameHeight = [kDBSizedCharacters sizeWithFont:[UIFont fontWithName:[parametersDictionary objectForKey:kDBFontNameKey] size:(maximumFontPointSize + 0.1)]].height;
            maximumFontPointSize += 0.1;
        }
        [parametersDictionary setObject:[UIFont fontWithName:[parametersDictionary objectForKey:kDBFontNameKey] size:maximumFontPointSize] forKey:kDBFontKey];
        
        //Calculate Frame Width
        self.frame = frameTemplate;
        self.frame = [self frameForState:kDBStateInitial];
        
        //Set initial parameters to button
        [self setTitle:[[parametersDictionary objectForKey:kDBStateInitial] objectForKey:kDBParameterTitleKey] forState:UIControlStateNormal];
        [self setTitleColor:[UIColor whiteColor] forState:UIControlStateNormal];
        [self setTitleShadowColor:[UIColor colorWithWhite:0 alpha:0.5] forState:UIControlStateNormal];
        self.titleLabel.textAlignment = UITextAlignmentCenter;
        self.titleLabel.shadowOffset = CGSizeMake(0, -1);
        self.titleLabel.backgroundColor = [UIColor clearColor];
        self.titleLabel.font = [parametersDictionary objectForKey:kDBFontKey];
        
        self.contentMode = UIViewContentModeCenter;
        
        //Set current state
        currentState = kDBStateInitial;
        
        [self setupLayers];
    }
    return self;
}

- (void)setupLayers{
    
    CAGradientLayer *bevelLayer = [CAGradientLayer layer];
    bevelLayer.frame = CGRectMake(0, 0, CGRectGetWidth(self.frame), CGRectGetHeight(self.frame));
    bevelLayer.colors = [NSArray arrayWithObjects:(id)[UIColor colorWithWhite:0 alpha:0.5].CGColor, [UIColor whiteColor].CGColor, nil];
    bevelLayer.cornerRadius = 4.0;
    bevelLayer.needsDisplayOnBoundsChange = YES;
    
    colorLayer = [CALayer layer];
    colorLayer.frame = CGRectMake(0, 1, CGRectGetWidth(self.frame), CGRectGetHeight(self.frame)-2);
    colorLayer.borderColor = [UIColor colorWithWhite:0 alpha:0.1].CGColor;
    UIColor *tint = [[parametersDictionary objectForKey:kDBStateInitial] objectForKey:kDBParameterTintKey];
    colorLayer.backgroundColor = tint.CGColor;
    colorLayer.borderWidth = 1.0;
    colorLayer.cornerRadius = 4.0;
    colorLayer.needsDisplayOnBoundsChange = YES;
    
    CAGradientLayer *colorGradient = [CAGradientLayer layer];
    colorGradient.frame = CGRectMake(0, 1, CGRectGetWidth(self.frame), CGRectGetHeight(self.frame)-2);
    colorGradient.colors = [NSArray arrayWithObjects:(id)[UIColor colorWithWhite:1 alpha:0.1].CGColor, [UIColor colorWithWhite:0.2 alpha:0.1].CGColor , nil];
    colorGradient.locations = [NSArray arrayWithObjects:[NSNumber numberWithFloat:0.0], [NSNumber numberWithFloat:1.0], nil];
    colorGradient.cornerRadius = 4.0;
    colorGradient.needsDisplayOnBoundsChange = YES;
    
    [self.layer addSublayer:bevelLayer];
    [self.layer addSublayer:colorLayer];
    [self.layer addSublayer:colorGradient];
    [self bringSubviewToFront:self.titleLabel];
    [self bringSubviewToFront:self.imageView];
    
}

- (void)toggleState
{
    //Easy use toggle method, do not access outside, use changeButtonToState:. You must use changeButtonToState to move from processing to kDBStateSuccess, kDBStateFailure, or kDBStateInitial
    //Initial State
    if ([currentState isEqualToString:kDBStateInitial] && [parametersDictionary objectForKey:kDBStateConfirm] != nil) { //To confirm
        [self changeButtonToState:kDBStateConfirm];
    } else if ([currentState isEqualToString:kDBStateInitial] && [parametersDictionary objectForKey:kDBStateConfirm] == nil && [parametersDictionary objectForKey:kDBStateProcess] != nil) { //To process
        [self changeButtonToState:kDBStateProcess];
        [self sendActionsForControlEvents:UIControlEventTouchUpInside];
    } else if ([currentState isEqualToString:kDBStateInitial] && [parametersDictionary objectForKey:kDBStateConfirm] == nil && [parametersDictionary objectForKey:kDBStateProcess] == nil && [parametersDictionary objectForKey:kDBStateSuccess] != nil && [parametersDictionary objectForKey:kDBStateFailure] != nil) { //To success/failure
        [self sendActionsForControlEvents:UIControlEventTouchUpInside];
    } else if ([currentState isEqualToString:kDBStateInitial] && [parametersDictionary objectForKey:kDBStateConfirm] == nil && [parametersDictionary objectForKey:kDBStateProcess] == nil  && [parametersDictionary objectForKey:kDBStateSuccess] == nil && [parametersDictionary objectForKey:kDBStateFailure] == nil && [parametersDictionary objectForKey:kDBStateDisabled] != nil) { //To disabled
        [self changeButtonToState:kDBStateDisabled];
        [self sendActionsForControlEvents:UIControlEventTouchUpInside];
    } else if ([currentState isEqualToString:kDBStateInitial] && [parametersDictionary objectForKey:kDBStateConfirm] == nil && [parametersDictionary objectForKey:kDBStateProcess] == nil && [parametersDictionary objectForKey:kDBStateSuccess] == nil && [parametersDictionary objectForKey:kDBStateFailure] == nil && [parametersDictionary objectForKey:kDBStateDisabled] == nil) { //To Initial
        [self sendActionsForControlEvents:UIControlEventTouchUpInside];
    }
    //Confirm State
    else if ([currentState isEqualToString:kDBStateConfirm] && [parametersDictionary objectForKey:kDBStateProcess] != nil) { //To Process
        [self sendActionsForControlEvents:UIControlEventTouchUpInside];
        [self changeButtonToState:kDBStateProcess];
    } else if ([currentState isEqualToString:kDBStateConfirm] && [parametersDictionary objectForKey:kDBStateProcess] == nil && [parametersDictionary objectForKey:kDBStateSuccess] != nil && [parametersDictionary objectForKey:kDBStateFailure] != nil) {//To success/failure
        [self sendActionsForControlEvents:UIControlEventTouchUpInside];
    } else if ([currentState isEqualToString:kDBStateConfirm] && [parametersDictionary objectForKey:kDBStateProcess] == nil && [parametersDictionary objectForKey:kDBStateSuccess] == nil && [parametersDictionary objectForKey:kDBStateFailure] == nil && [parametersDictionary objectForKey:kDBStateDisabled] != nil) { //To Disabled
        [self sendActionsForControlEvents:UIControlEventTouchUpInside];
        [self changeButtonToState:kDBStateDisabled];
    } else if ([currentState isEqualToString:kDBStateConfirm] && [parametersDictionary objectForKey:kDBStateProcess] == nil && [parametersDictionary objectForKey:kDBStateSuccess] == nil && [parametersDictionary objectForKey:kDBStateFailure] == nil && [parametersDictionary objectForKey:kDBStateDisabled] == nil) { //To Initial
        [self sendActionsForControlEvents:UIControlEventTouchUpInside];
        [self changeButtonToState:kDBStateInitial];
    }
    //Success/Failure State
    else if (([currentState isEqualToString:kDBStateFailure] || [currentState isEqualToString:kDBStateSuccess]) && [parametersDictionary objectForKey:kDBStateDisabled] != nil) {//To Disabled
        [self changeButtonToState:kDBStateDisabled];
    } else if (([currentState isEqualToString:kDBStateFailure] || [currentState isEqualToString:kDBStateSuccess]) && [parametersDictionary objectForKey:kDBStateDisabled] == nil) {//To Initial
        [self changeButtonToState:kDBStateInitial];
    }
}

- (void)changeButtonToState:(NSString *)state
{
    if (![state isEqualToString:currentState]) {
        self.userInteractionEnabled = NO;
        //Hide content which doesn't animate with frame change
        self.titleLabel.alpha = 0;
        [self setImage:nil forState:UIControlStateNormal];
        
        CGSize newSize;
        UIColor *newTintColor;
        
        //Set title, new frame size and new tint color
        if ([state isEqualToString:kDBStateInitial]) {
            [self setTitle:[[parametersDictionary objectForKey:kDBStateInitial] objectForKey:kDBParameterTitleKey] forState:UIControlStateNormal];
            [self setTitleColor:[UIColor whiteColor] forState:UIControlStateNormal];
            [self setTitleShadowColor:[UIColor colorWithWhite:0 alpha:0.5] forState:UIControlStateNormal];
            self.titleLabel.shadowOffset = CGSizeMake(0, -1);
            newSize = [self frameForState:kDBStateInitial].size;
            newTintColor = [[parametersDictionary objectForKey:kDBStateInitial] objectForKey:kDBParameterTintKey];
            self.imageView.layer.shadowOffset = CGSizeMake(0, -1);
            self.imageView.layer.shadowColor = [UIColor blackColor].CGColor;
        } else if ([state isEqualToString:kDBStateConfirm]) {
            [self setTitle:[[parametersDictionary objectForKey:kDBStateConfirm] objectForKey:kDBParameterTitleKey] forState:UIControlStateNormal];
            [self setTitleColor:[UIColor whiteColor] forState:UIControlStateNormal];
            [self setTitleShadowColor:[UIColor colorWithWhite:0 alpha:0.5] forState:UIControlStateNormal];
            self.titleLabel.shadowOffset = CGSizeMake(0, -1);
            newSize = [self frameForState:kDBStateConfirm].size;
            newTintColor = [[parametersDictionary objectForKey:kDBStateConfirm] objectForKey:kDBParameterTintKey];
            self.imageView.layer.shadowOffset = CGSizeMake(0, -1);
            self.imageView.layer.shadowColor = [UIColor blackColor].CGColor;
        } else if ([state isEqualToString:kDBStateProcess]) {
            [self setTitle:[[parametersDictionary objectForKey:kDBStateProcess] objectForKey:kDBParameterTitleKey] forState:UIControlStateNormal];
            [self setTitleColor:[UIColor whiteColor] forState:UIControlStateNormal];
            [self setTitleShadowColor:[UIColor colorWithWhite:0 alpha:0.5] forState:UIControlStateNormal];
            self.titleLabel.shadowOffset = CGSizeMake(0, -1);
            newSize = [self frameForState:kDBStateProcess].size;
            newTintColor = [[parametersDictionary objectForKey:kDBStateProcess] objectForKey:kDBParameterTintKey];
            self.imageView.layer.shadowOffset = CGSizeMake(0, -1);
            self.imageView.layer.shadowColor = [UIColor blackColor].CGColor;
        }  else if ([state isEqualToString:kDBStateSuccess]) {
            [self setTitle:[[parametersDictionary objectForKey:kDBStateSuccess] objectForKey:kDBParameterTitleKey] forState:UIControlStateNormal];
            [self setTitleColor:[UIColor whiteColor] forState:UIControlStateNormal];
            [self setTitleShadowColor:[UIColor colorWithWhite:0 alpha:0.5] forState:UIControlStateNormal];
            self.titleLabel.shadowOffset = CGSizeMake(0, -1);
            newSize = [self frameForState:kDBStateSuccess].size;
            newTintColor = [[parametersDictionary objectForKey:kDBStateSuccess] objectForKey:kDBParameterTintKey];
            self.imageView.layer.shadowOffset = CGSizeMake(0, -1);
            self.imageView.layer.shadowColor = [UIColor blackColor].CGColor;
        }  else if ([state isEqualToString:kDBStateFailure]) {
            [self setTitle:[[parametersDictionary objectForKey:kDBStateFailure] objectForKey:kDBParameterTitleKey] forState:UIControlStateNormal];
            [self setTitleColor:[UIColor whiteColor] forState:UIControlStateNormal];
            [self setTitleShadowColor:[UIColor colorWithWhite:0 alpha:0.5] forState:UIControlStateNormal];
            self.titleLabel.shadowOffset = CGSizeMake(0, -1);
            newSize = [self frameForState:kDBStateFailure].size;
            newTintColor = [[parametersDictionary objectForKey:kDBStateFailure] objectForKey:kDBParameterTintKey];
            self.imageView.layer.shadowOffset = CGSizeMake(0, -1);
            self.imageView.layer.shadowColor = [UIColor blackColor].CGColor;
        }   else if ([state isEqualToString:kDBStateDisabled]) {
            [self setTitle:[[parametersDictionary objectForKey:kDBStateDisabled] objectForKey:kDBParameterTitleKey] forState:UIControlStateNormal];
            [self setTitleColor:[UIColor colorWithWhite:0.6 alpha:1] forState:UIControlStateNormal];
            [self setTitleShadowColor:[UIColor colorWithWhite:1 alpha:1] forState:UIControlStateNormal];
            self.titleLabel.shadowOffset = CGSizeMake(0, 1);
            newSize = [self frameForState:kDBStateDisabled].size;
            newTintColor = [[parametersDictionary objectForKey:kDBStateDisabled] objectForKey:kDBParameterTintKey];
            self.imageView.layer.shadowOffset = CGSizeMake(0, 1);
            self.imageView.layer.shadowColor = [UIColor colorWithWhite:1 alpha:1].CGColor;
        }
        
        //Calculate offset
        CGFloat offset = newSize.width - self.frame.size.width;
        
        [CATransaction begin];
        [CATransaction setAnimationDuration:0.25];
        [CATransaction setCompletionBlock:^{
            //Set new frame and animate
            CGRect frameRect = self.frame;
            if ([[parametersDictionary objectForKey:kDBToggleStyleKey] isEqualToString:kDBToggleLeft]) {
                frameRect.origin.x = frameRect.origin.x - offset;
            } else if ([[parametersDictionary objectForKey:kDBToggleStyleKey] isEqualToString:kDBToggleRight]) {
                //Do nothing
            } else if ([[parametersDictionary objectForKey:kDBToggleStyleKey] isEqualToString:kDBToggleCenter]) {
                frameRect.origin.x = frameRect.origin.x - (offset / 2.0);
            }
            frameRect.size.width = frameRect.size.width + offset;
            self.frame = frameRect;
            
            [CATransaction setDisableActions:YES];
            [CATransaction setCompletionBlock:^{
                //allow userInteraction on Initial State and Confirm State
                if ([state isEqualToString:kDBStateInitial] || [state isEqualToString:kDBStateConfirm]) {
                    self.userInteractionEnabled = YES;
                }
            }];
            //Manage Sub Layers
            for (CALayer *layer in self.layer.sublayers) {
                //Set new frame and animate
                CGRect frameRect = layer.frame;
                if ([[parametersDictionary objectForKey:kDBToggleStyleKey] isEqualToString:kDBToggleLeft]) {
                    frameRect.origin.x = frameRect.origin.x + offset;
                } else if ([[parametersDictionary objectForKey:kDBToggleStyleKey] isEqualToString:kDBToggleRight]) {
                    //Do nothing
                } else if ([[parametersDictionary objectForKey:kDBToggleStyleKey] isEqualToString:kDBToggleCenter]) {
                    frameRect.origin.x = frameRect.origin.x + (offset / 2.0);
                }
                layer.frame = frameRect;
            }
            
            [CATransaction commit];
            
            //Show content
            self.titleLabel.alpha = 1;
            if ([parametersDictionary objectForKey:kDBIconKey] != nil && [currentState isEqualToString:kDBStateDisabled]) {
                [self setImage:[self colorizeImage:[parametersDictionary objectForKey:kDBIconKey] color:[UIColor colorWithWhite:0.6 alpha:1]] forState:UIControlStateNormal];
            } else {
                [self setImage:[self colorizeImage:[parametersDictionary objectForKey:kDBIconKey] color:[UIColor whiteColor]] forState:UIControlStateNormal];
            }
            [self setNeedsLayout];
        }];
        
        //Animate color change
        CABasicAnimation *colorAnimation = [CABasicAnimation animationWithKeyPath:@"backgroundColor"];
        colorAnimation.removedOnCompletion = NO;
        colorAnimation.fillMode = kCAFillModeForwards;
        
        UIColor *fromColor = [[parametersDictionary objectForKey:currentState] objectForKey:kDBParameterTintKey];
        UIColor *toColor = [[parametersDictionary objectForKey:state] objectForKey:kDBParameterTintKey];
        colorAnimation.fromValue = (id)fromColor.CGColor;
        colorAnimation.toValue = (id)toColor.CGColor;
        
        [colorLayer addAnimation:colorAnimation forKey:@"colorAnimation"];
        
        //Animate Layer Scale
        for (CALayer *layer in self.layer.sublayers) {
            CGRect rect = layer.frame;
            if ([[parametersDictionary objectForKey:kDBToggleStyleKey] isEqualToString:kDBToggleLeft]) {
                rect.origin.x = rect.origin.x - offset;
            } else if ([[parametersDictionary objectForKey:kDBToggleStyleKey] isEqualToString:kDBToggleRight]) {
                //Do nothing
            } else if ([[parametersDictionary objectForKey:kDBToggleStyleKey] isEqualToString:kDBToggleCenter]) {
                rect.origin.x = rect.origin.x - (offset / 2);
            }
            rect.size.width = rect.size.width + offset;
            layer.frame = rect;
        }
        
        [CATransaction commit];
        [self setNeedsDisplay];
        
        //If success or failure, change state after notification
        if ([state isEqualToString:kDBStateSuccess] || [state isEqualToString:kDBStateFailure]) {
            [self performSelector:@selector(toggleState) withObject:nil afterDelay:kDBAfterProcessTimer];
        }
        
        currentState = state;
    }
}

- (UIImage *)colorizeImage:(UIImage *)baseImage color:(UIColor *)theColor {
    UIGraphicsBeginImageContext(baseImage.size);
    
    CGContextRef ctx = UIGraphicsGetCurrentContext();
    CGRect area = CGRectMake(0, 0, baseImage.size.width, baseImage.size.height);
    
    CGContextScaleCTM(ctx, 1, -1);
    CGContextTranslateCTM(ctx, 0, -area.size.height);
    
    CGContextSaveGState(ctx);
    CGContextClipToMask(ctx, area, baseImage.CGImage);
    
    [theColor set];
    CGContextFillRect(ctx, area);
    
    CGContextRestoreGState(ctx);
    
    CGContextSetBlendMode(ctx, kCGBlendModeMultiply);
    
    CGContextDrawImage(ctx, area, baseImage.CGImage);
    
    UIImage *newImage = UIGraphicsGetImageFromCurrentImageContext();
    
    UIGraphicsEndImageContext();
    
    return newImage;
}

- (void)disableWithDisabledStateParameters:(NSDictionary *)parameters
{
    [parametersDictionary setObject:[self preformPrameterCheck:parameters forState:kDBStateDisabled] forKey:kDBStateDisabled];
    [self changeButtonToState:kDBStateDisabled];
}

- (void)darken{
    darkenLayer = [CALayer layer];
    darkenLayer.frame = CGRectMake(0, 0, CGRectGetWidth(self.frame), CGRectGetHeight(self.frame));
    darkenLayer.backgroundColor = [UIColor colorWithWhite:0 alpha:0.2].CGColor;
    darkenLayer.cornerRadius = 4.0;
    darkenLayer.needsDisplayOnBoundsChange = YES;
    [self.layer addSublayer:darkenLayer];
}

- (void)lighten{
    if(darkenLayer){
        [darkenLayer removeFromSuperlayer];
        darkenLayer = nil;
    }
}

- (void)cancel{
    [cancelOverlay removeFromSuperview];
    cancelOverlay = nil;
    [self changeButtonToState:kDBStateInitial];
}

- (void)touchesBegan:(NSSet *)touches withEvent:(UIEvent *)event
{
    //Only allow touch events on first click or confirm
    if(![currentState isEqualToString:kDBStateDisabled] && ![currentState isEqualToString:kDBStateProcess] && ![currentState isEqualToString:kDBStateSuccess] && ![currentState isEqualToString:kDBStateFailure] && self.userInteractionEnabled){
        [self darken];
    }
    [super touchesBegan:touches withEvent:event];
}

- (void)touchesEnded:(NSSet *)touches withEvent:(UIEvent *)event
{
    if(![currentState isEqualToString:kDBStateDisabled] && ![currentState isEqualToString:kDBStateProcess] && ![currentState isEqualToString:kDBStateSuccess] && ![currentState isEqualToString:kDBStateFailure] && self.userInteractionEnabled){
        if (!CGRectContainsPoint(self.frame, [[touches anyObject] locationInView:self.superview])) {
            //Cancel Confirm. TouchUpOutside
            [self lighten];
            [super touchesCancelled:touches withEvent:event];
        } else if ([currentState isEqualToString:kDBStateConfirm]) {
            [self lighten];
            [cancelOverlay removeFromSuperview];
            cancelOverlay = nil;
            [self toggleState];
            [super touchesCancelled:touches withEvent:event]; //Taken care of in toggleState
        } else if ([currentState isEqualToString:kDBStateInitial]) {
            [self lighten];
            [self toggleState];
            if (!cancelOverlay && [parametersDictionary objectForKey:kDBStateConfirm] != nil) {
                cancelOverlay = [UIButton buttonWithType:UIButtonTypeCustom];
                [cancelOverlay setFrame:[UIScreen mainScreen].applicationFrame];
                [cancelOverlay addTarget:self action:@selector(cancel) forControlEvents:UIControlEventTouchDown];
                [self.superview addSubview:cancelOverlay];
            }
            [self.superview bringSubviewToFront:self];
        }
    }
}

@end
